+{
   entry => <<'ENTRY', # for DUCET v6.0.0
0491      ; [.1964.0020.0002.0491] # CYRILLIC SMALL LETTER GHE WITH UPTURN
0490      ; [.1964.0020.0008.0490] # CYRILLIC CAPITAL LETTER GHE WITH UPTURN
ENTRY
};
