+{
   entry => <<'ENTRY', # for DUCET v6.0.0
006E 0067 ; [.1703.0020.0002.006E] # <LATIN SMALL LETTER N, LATIN SMALL LETTER G>
004E 0067 ; [.1703.0020.0007.004E] # <LATIN CAPITAL LETTER N, LATIN SMALL LETTER G>
004E 0047 ; [.1703.0020.0008.004E] # <LATIN CAPITAL LETTER N, LATIN CAPITAL LETTER G>
014B      ; [.1703.0020.001C.014B] # LATIN SMALL LETTER ENG
014A      ; [.1703.0020.001D.014A] # LATIN CAPITAL LETTER ENG
02BB      ; [.1845.0020.0002.02BB] # MODIFIER LETTER TURNED COMMA
02BD      ; [.1845.0020.0003.02BD] # MODIFIER LETTER REVERSED COMMA
0101      ; [.15A3.0021.0002.0101] # LATIN SMALL LETTER A WITH MACRON
0061 0304 ; [.15A3.0021.0002.0101] # LATIN SMALL LETTER A WITH MACRON
00E1      ; [.15A3.0021.0003.00E1] # LATIN SMALL LETTER A WITH ACUTE
0061 0301 ; [.15A3.0021.0003.00E1] # LATIN SMALL LETTER A WITH ACUTE
0061 0341 ; [.15A3.0021.0003.00E1] # LATIN SMALL LETTER A WITH ACUTE
0100      ; [.15A3.0021.0008.0100] # LATIN CAPITAL LETTER A WITH MACRON
0041 0304 ; [.15A3.0021.0008.0100] # LATIN CAPITAL LETTER A WITH MACRON
00C1      ; [.15A3.0021.0009.00C1] # LATIN CAPITAL LETTER A WITH ACUTE
0041 0301 ; [.15A3.0021.0009.00C1] # LATIN CAPITAL LETTER A WITH ACUTE
0041 0341 ; [.15A3.0021.0009.00C1] # LATIN CAPITAL LETTER A WITH ACUTE
0113      ; [.15FF.0021.0002.0113] # LATIN SMALL LETTER E WITH MACRON
0065 0304 ; [.15FF.0021.0002.0113] # LATIN SMALL LETTER E WITH MACRON
00E9      ; [.15FF.0021.0003.00E9] # LATIN SMALL LETTER E WITH ACUTE
0065 0301 ; [.15FF.0021.0003.00E9] # LATIN SMALL LETTER E WITH ACUTE
0065 0341 ; [.15FF.0021.0003.00E9] # LATIN SMALL LETTER E WITH ACUTE
0112      ; [.15FF.0021.0008.0112] # LATIN CAPITAL LETTER E WITH MACRON
0045 0304 ; [.15FF.0021.0008.0112] # LATIN CAPITAL LETTER E WITH MACRON
00C9      ; [.15FF.0021.0009.00C9] # LATIN CAPITAL LETTER E WITH ACUTE
0045 0301 ; [.15FF.0021.0009.00C9] # LATIN CAPITAL LETTER E WITH ACUTE
0045 0341 ; [.15FF.0021.0009.00C9] # LATIN CAPITAL LETTER E WITH ACUTE
012B      ; [.1680.0021.0002.012B] # LATIN SMALL LETTER I WITH MACRON
0069 0304 ; [.1680.0021.0002.012B] # LATIN SMALL LETTER I WITH MACRON
00ED      ; [.1680.0021.0003.00ED] # LATIN SMALL LETTER I WITH ACUTE
0069 0301 ; [.1680.0021.0003.00ED] # LATIN SMALL LETTER I WITH ACUTE
0069 0341 ; [.1680.0021.0003.00ED] # LATIN SMALL LETTER I WITH ACUTE
012A      ; [.1680.0021.0008.012A] # LATIN CAPITAL LETTER I WITH MACRON
0049 0304 ; [.1680.0021.0008.012A] # LATIN CAPITAL LETTER I WITH MACRON
00CD      ; [.1680.0021.0009.00CD] # LATIN CAPITAL LETTER I WITH ACUTE
0049 0301 ; [.1680.0021.0009.00CD] # LATIN CAPITAL LETTER I WITH ACUTE
0049 0341 ; [.1680.0021.0009.00CD] # LATIN CAPITAL LETTER I WITH ACUTE
014D      ; [.1724.0021.0002.014D] # LATIN SMALL LETTER O WITH MACRON
006F 0304 ; [.1724.0021.0002.014D] # LATIN SMALL LETTER O WITH MACRON
00F3      ; [.1724.0021.0003.00F3] # LATIN SMALL LETTER O WITH ACUTE
006F 0301 ; [.1724.0021.0003.00F3] # LATIN SMALL LETTER O WITH ACUTE
006F 0341 ; [.1724.0021.0003.00F3] # LATIN SMALL LETTER O WITH ACUTE
014C      ; [.1724.0021.0008.014C] # LATIN CAPITAL LETTER O WITH MACRON
004F 0304 ; [.1724.0021.0008.014C] # LATIN CAPITAL LETTER O WITH MACRON
00D3      ; [.1724.0021.0009.00D3] # LATIN CAPITAL LETTER O WITH ACUTE
004F 0301 ; [.1724.0021.0009.00D3] # LATIN CAPITAL LETTER O WITH ACUTE
004F 0341 ; [.1724.0021.0009.00D3] # LATIN CAPITAL LETTER O WITH ACUTE
016B      ; [.17E9.0021.0002.016B] # LATIN SMALL LETTER U WITH MACRON
0075 0304 ; [.17E9.0021.0002.016B] # LATIN SMALL LETTER U WITH MACRON
00FA      ; [.17E9.0021.0003.00FA] # LATIN SMALL LETTER U WITH ACUTE
0075 0301 ; [.17E9.0021.0003.00FA] # LATIN SMALL LETTER U WITH ACUTE
0075 0341 ; [.17E9.0021.0003.00FA] # LATIN SMALL LETTER U WITH ACUTE
016A      ; [.17E9.0021.0008.016A] # LATIN CAPITAL LETTER U WITH MACRON
0055 0304 ; [.17E9.0021.0008.016A] # LATIN CAPITAL LETTER U WITH MACRON
00DA      ; [.17E9.0021.0009.00DA] # LATIN CAPITAL LETTER U WITH ACUTE
0055 0301 ; [.17E9.0021.0009.00DA] # LATIN CAPITAL LETTER U WITH ACUTE
0055 0341 ; [.17E9.0021.0009.00DA] # LATIN CAPITAL LETTER U WITH ACUTE
ENTRY
};
